@@ -6,6 +6,9 @@ use std::collections::{BTreeMap, HashMap, HashSet};
 use utoipa::ToSchema;
 // use csv::Writer;
 // use std::error::Error;
+use crate::functions::reactions::{
+    compose_support_reaction_vector_equilibrium, extract_reaction_nodes,
+};
 use crate::models::imperfections::imperfectioncase::ImperfectionCase;
 use crate::models::loads::loadcase::LoadCase;
 use crate::models::loads::loadcombination::LoadCombination;
@@ -28,7 +31,6 @@ use crate::functions::hinge_and_release_operations::{
 use crate::functions::load_assembler::{
     assemble_distributed_loads, assemble_nodal_loads, assemble_nodal_moments,
 };
-use crate::functions::reactions::extract_reaction_nodes;
 
 #[derive(Serialize, Deserialize, ToSchema, Debug)]
 pub struct FERS {
@@ -82,33 +84,6 @@ macro_rules! get_case_insensitive {
 }
 
 impl FERS {
-    fn compute_full_residual_with_rigid_constraints(
-        &self,
-        active_member_map: &std::collections::HashMap<u32, bool>,
-        displacement_vector_full: &nalgebra::DMatrix<f64>,
-        external_load_vector_full: &nalgebra::DMatrix<f64>,
-    ) -> Result<nalgebra::DMatrix<f64>, String> {
-        // Build unconstrained full operator (material + support springs; no geometric part here)
-        let stiffness_matrix_full = self.build_operator_with_supports(active_member_map, None)?;
-
-        // Build the same elimination used during the solve (rigid links → MPC)
-        let elimination = self.build_rigid_elimination_partial_using_hinges()?;
-
-        // Reduce
-        let displacement_vector_reduced = elimination.s.transpose() * displacement_vector_full;
-        let external_load_vector_reduced = elimination.s.transpose() * external_load_vector_full;
-        let stiffness_matrix_reduced =
-            elimination.s.transpose() * stiffness_matrix_full * &elimination.s;
-
-        // Residual in reduced coordinates
-        let residual_vector_reduced =
-            stiffness_matrix_reduced * displacement_vector_reduced - external_load_vector_reduced;
-
-        // Map residual back to full space
-        let residual_vector_full = &elimination.s * residual_vector_reduced;
-        Ok(residual_vector_full)
-    }
-
     fn visit_unique_supported_nodes<F>(&self, mut visitor: F) -> Result<(), String>
     where
         F: FnMut(u32, usize, &NodalSupport) -> Result<(), String>,
@@ -159,143 +134,6 @@ impl FERS {
         Ok(k)
     }
 
-    fn compute_support_spring_reaction_vector_verbose(
-        &self,
-        displacement_vector_full: &nalgebra::DMatrix<f64>,
-    ) -> Result<nalgebra::DMatrix<f64>, String> {
-        let number_of_degrees_of_freedom = self.compute_num_dofs();
-        let mut spring_reaction_vector_full =
-            nalgebra::DMatrix::<f64>::zeros(number_of_degrees_of_freedom, 1);
-
-        self.visit_unique_supported_nodes(|_node_id, base_index, nodal_support| {
-            // Translational springs
-            for (axis_label, local_dof) in TRANSLATION_AXES {
-                if let Some(condition) =
-                    get_case_insensitive!(nodal_support.displacement_conditions, axis_label)
-                {
-                    if let SupportConditionType::Spring = condition.condition_type {
-                        let stiffness = Self::spring_stiffness_or_error(
-                            nodal_support.id,
-                            axis_label,
-                            "displacement",
-                            condition.stiffness,
-                        )?;
-                        let u = displacement_vector_full[(base_index + local_dof, 0)];
-                        spring_reaction_vector_full[(base_index + local_dof, 0)] += stiffness * u;
-                    }
-                }
-            }
-
-            // Rotational springs
-            for (axis_label, local_dof) in ROTATION_AXES {
-                if let Some(condition) =
-                    get_case_insensitive!(nodal_support.rotation_conditions, axis_label)
-                {
-                    if let SupportConditionType::Spring = condition.condition_type {
-                        let stiffness = Self::spring_stiffness_or_error(
-                            nodal_support.id,
-                            axis_label,
-                            "rotation",
-                            condition.stiffness,
-                        )?;
-                        let r = displacement_vector_full[(base_index + local_dof, 0)];
-                        spring_reaction_vector_full[(base_index + local_dof, 0)] += stiffness * r;
-                    }
-                }
-            }
-            Ok(())
-        })?; // propagate visitor errors
-
-        Ok(spring_reaction_vector_full)
-    }
-
-    /// Compose a support reaction vector with the following semantics:
-    /// - Fixed  DOFs: reaction = residual = (K u − f) at that DOF
-    /// - Spring DOFs: reaction = k * dof_value
-    /// - Free   DOFs: reaction = 0
-    /// - PositiveOnly: clamp to ≥ 0
-    /// - NegativeOnly: clamp to ≤ 0
-    fn compose_support_reaction_vector(
-        &self,
-        active_member_map: &std::collections::HashMap<u32, bool>,
-        displacement_vector_full: &nalgebra::DMatrix<f64>,
-        external_load_vector_full: &nalgebra::DMatrix<f64>,
-    ) -> Result<nalgebra::DMatrix<f64>, String> {
-        let residual_vector_full = self.compute_full_residual_with_rigid_constraints(
-            active_member_map,
-            displacement_vector_full,
-            external_load_vector_full,
-        )?;
-
-        // Spring contributions
-        let spring_reaction_vector_full =
-            self.compute_support_spring_reaction_vector_verbose(displacement_vector_full)?;
-
-        let base_index_node1 = (1usize - 1) * 6;
-        log::debug!(
-            "node 1 residual Fx..Mz = {:?}",
-            residual_vector_full.view((base_index_node1, 0), (6, 1))
-        );
-        log::debug!(
-            "node 1 spring   Fx..Mz = {:?}",
-            spring_reaction_vector_full.view((base_index_node1, 0), (6, 1))
-        );
-
-        let total_number_of_degrees_of_freedom = self.compute_num_dofs();
-        let mut composed_reaction_vector_full =
-            nalgebra::DMatrix::<f64>::zeros(total_number_of_degrees_of_freedom, 1);
-
-        self.visit_unique_supported_nodes(|_node_id, base_index, support| {
-            // Translations
-            for (axis_label, local_dof) in TRANSLATION_AXES {
-                let value = match get_case_insensitive!(support.displacement_conditions, axis_label)
-                    .map(|condition| &condition.condition_type)
-                {
-                    Some(SupportConditionType::Free) => 0.0,
-                    Some(SupportConditionType::Spring) => {
-                        spring_reaction_vector_full[(base_index + local_dof, 0)]
-                    }
-                    Some(SupportConditionType::PositiveOnly) => {
-                        residual_vector_full[(base_index + local_dof, 0)].max(0.0)
-                    }
-                    Some(SupportConditionType::NegativeOnly) => {
-                        residual_vector_full[(base_index + local_dof, 0)].min(0.0)
-                    }
-                    None | Some(SupportConditionType::Fixed) => {
-                        residual_vector_full[(base_index + local_dof, 0)]
-                    }
-                };
-                composed_reaction_vector_full[(base_index + local_dof, 0)] = value;
-            }
-
-            // Rotations
-            for (axis_label, local_dof) in ROTATION_AXES {
-                let value = match get_case_insensitive!(support.rotation_conditions, axis_label)
-                    .map(|condition| &condition.condition_type)
-                {
-                    Some(SupportConditionType::Free) => 0.0,
-                    Some(SupportConditionType::Spring) => {
-                        spring_reaction_vector_full[(base_index + local_dof, 0)]
-                    }
-                    Some(SupportConditionType::PositiveOnly) => {
-                        residual_vector_full[(base_index + local_dof, 0)].max(0.0)
-                    }
-                    Some(SupportConditionType::NegativeOnly) => {
-                        residual_vector_full[(base_index + local_dof, 0)].min(0.0)
-                    }
-                    None | Some(SupportConditionType::Fixed) => {
-                        residual_vector_full[(base_index + local_dof, 0)]
-                    }
-                };
-                composed_reaction_vector_full[(base_index + local_dof, 0)] = value;
-            }
-
-            Ok(())
-        })?; // <- propagate visitor result
-
-        Ok(composed_reaction_vector_full)
-    }
-
     pub fn build_lookup_maps(
         &self,
     ) -> (
@@ -1076,7 +914,7 @@ impl FERS {
 
         // reactions from the linear operator in FULL space
         let composed_support_reaction_vector_full =
-            self.compose_support_reaction_vector(&active_map, &u_full, &load_vector_full)?;
+            compose_support_reaction_vector_equilibrium(self, &result_type, &u_full)?;
 
         let results = self
             .build_and_store_results(
@@ -1158,7 +996,7 @@ impl FERS {
 
         // Reactions from the linear operator (no geometric part), FULL space
         let composed_support_reaction_vector_full =
-            self.compose_support_reaction_vector(&active_map, &u_full, &load_vector_full)?;
+            compose_support_reaction_vector_equilibrium(self, &result_type, &u_full)?;
 
         let results = self
             .build_and_store_results(
@@ -1239,6 +1077,16 @@ impl FERS {
         )
     }
 
+    /// Compose support reactions by **equilibrium per node**, not from residuals.
+    /// - Deformable members → sum end forces AND end moments into their end nodes
+    /// - Add applied **nodal** loads and **nodal** moments (cases or combinations)
+    /// - Roll up wrenches along **rigid links** (F_a += F_b ; M_a += M_b + r × F_b)
+    /// - For each support DOF:
+    ///     Fixed           → reaction = −(net node wrench component)
+    ///     Spring          → reaction = k * dof_value   (existing spring logic)
+    ///     Free            → 0
+    ///     PositiveOnly    → clamp ≥ 0
+    ///     NegativeOnly    → clamp ≤ 0
     pub fn build_and_store_results(
         &mut self,
         name: String,
@@ -1246,13 +1094,11 @@ impl FERS {
         displacement_vector: &DMatrix<f64>,
         global_reaction_vector: &DMatrix<f64>,
     ) -> Result<&Results, String> {
-        use std::cmp::Reverse;
-
         // 1) Element/member results + basic node maps
         let member_results =
             compute_member_results_from_displacement(self, &result_type, displacement_vector);
         let displacement_nodes = extract_displacements(self, displacement_vector);
-        let mut reaction_nodes = extract_reaction_nodes(self, global_reaction_vector);
+        let reaction_nodes = extract_reaction_nodes(self, global_reaction_vector);
 
         // ---- member_id -> (start_node_id, end_node_id)
         let mut member_ends: HashMap<u32, (u32, u32)> = HashMap::new();
@@ -1301,41 +1147,6 @@ impl FERS {
             }
         }
 
-        fn depth_of(node: u32, parent: &HashMap<u32, (u32, (f64, f64, f64))>) -> usize {
-            let mut d = 0usize;
-            let mut p = node;
-            while let Some(&(pp, _)) = parent.get(&p) {
-                d += 1;
-                p = pp;
-            }
-            d
-        }
-
-        // 4) Rigid-arm correction:
-        rigid_edges.sort_by_key(|&(_a, b, _)| Reverse(depth_of(b, &parent)));
-
-        for (a, b, (rx, ry, rz)) in rigid_edges {
-            let (fx_b, fy_b, fz_b) = node_force.get(&b).copied().unwrap_or((0.0, 0.0, 0.0));
-            if fx_b == 0.0 && fy_b == 0.0 && fz_b == 0.0 {
-                continue;
-            }
-
-            let mx = ry * fz_b - rz * fy_b;
-            let my = rz * fx_b - rx * fz_b;
-            let mz = rx * fy_b - ry * fx_b;
-
-            if let Some(rnode) = reaction_nodes.get_mut(&a) {
-                rnode.nodal_forces.mx += mx;
-                rnode.nodal_forces.my += my;
-                rnode.nodal_forces.mz += mz;
-            } else {
-                let e = node_force.entry(a).or_insert((0.0, 0.0, 0.0));
-                e.0 += fx_b;
-                e.1 += fy_b;
-                e.2 += fz_b;
-            }
-        }
-
         // 5) Pack results & store
         let total_members: usize = self.member_sets.iter().map(|set| set.members.len()).sum();
         let total_supports: usize = self.nodal_supports.len();
